<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */

namespace SmebankPlugin\Form\Settings;

if (!defined('_PS_VERSION_')) {
    exit;
}

use AdminController;
use Configuration;
use Context;
use SmebankPlugin\Form\Settings\Subform\CheckoutSettingsSubform;
use SmebankPlugin\Form\Settings\Subform\MainSettingsSubform;
use SmebankPlugin\Form\Settings\Subform\SubformInterface;
use SmebankPlugin\Gateway;
use SmebankPlugin\GatewaySettings;
use SmebankPlugin\Utils\AdminHtmlHelper;
use SmebankPlugin\Utils\Translator;
use SmebankPluginFramework\Presentation\Admin\Form\BuildGuideline\CheckoutSettingsFormBuildGuideline;
use SmebankPluginFramework\Presentation\Admin\Form\BuildGuideline\MainSettingsFormBuildGuideline;
use SmebankPluginFramework\Presentation\Admin\Form\Validator\IbanRegexValidator;
use Tools;

class SettingsForm extends \HelperFormCore
{
    /**
     * @var \Smebank
     */
    public $module;
    /**
     * @var SubformInterface[]
     */
    private $subforms;

    /**
     * @var array
     */
    private $validationErrors = [];

    /**
     * @param \Smebank $module
     * @param Gateway|null $gateway
     */
    public function __construct(\Smebank $module)
    {
        $this->module = $module;
        $this->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');
        $this->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ?: 0;
        $this->show_toolbar = false;
        $this->name_controller = $this->module->name;
        $this->submit_action = 'submitKevinModule';
        $this->currentIndex = AdminController::$currentIndex . '&configure=' . $this->module->name;
        $this->token = Tools::getAdminTokenLite('AdminModules');
        $this->subforms = [
            (new MainSettingsSubform(new MainSettingsFormBuildGuideline())),
            (new CheckoutSettingsSubform(new CheckoutSettingsFormBuildGuideline())),
        ];

        $this->populate();

        parent::__construct();
    }

    /**
     * @param Gateway|null $gateway
     *
     * @return string
     *
     * @throws \SmartyException
     */
    public function html($gateway)
    {
        Context::getContext()->controller->addCSS($this->module->getPathUri() . '/views/css/back.css');
        Context::getContext()->controller->addJS($this->module->getPathUri() . '/views/js/admin_settings.js');
        Context::getContext()->smarty->assign([
            'module_dir' => $this->module->getPathUri(),
            'text_website' => Translator::translate('Website'),
            'text_support' => Translator::translate('Support')
        ]);

        $html = Context::getContext()->smarty->fetch($this->module->getLocalPath() . 'views/templates/admin/configure.tpl');

        $placeholders = [];
        $subforms = [];
        foreach ($this->subforms as $subform) {
            foreach ($subform->getFormFields() as $formField) {
                if ($formField->getPlaceholder()) {
                    $placeholders[$formField->getName()] = $formField->getPlaceholder();
                }
            }
            $subforms[] = $subform->toArray();
        }
        \Media::addJsDef(['kevin_settings_placeholders' => $placeholders]);

        if ($this->isSubmitted()) {
            if (!$this->validationErrors) {
                $html .= AdminHtmlHelper::htmlSuccess(Translator::translate('Your settings have been saved.'));
            }
            foreach ($this->validationErrors as $validationError) {
                $html .= AdminHtmlHelper::htmlError($validationError);
            }
        }

        if ($gateway) {
            $html .= $this->getGatewayInfo($gateway);
        }

        return $html . $this->generateForm($subforms);
    }

    public function save()
    {
        foreach (GatewaySettings::getAllKeys() as $key) {
            $value = Tools::getValue($key);
            if ($value !== false) {
                \Configuration::updateValue($key, $value);
            }
        }
    }

    /**
     * @return bool
     */
    public function isSubmitted()
    {
        foreach ($this->subforms as $subform) {
            if (Tools::isSubmit($subform->getSubmitName())) {
                return true;
            }
        }

        return false;
    }

    /**
     * @return bool
     *
     * @throws \Exception
     */
    public function validate()
    {
        $this->validationErrors = [];
        foreach ($this->subforms as $subform) {
            foreach ($subform->getFormFields() as $formField) {
                $value = Tools::getValue($formField->getName());
                if ($formField->isRequired() && !$value) {
                    $this->validationErrors[] = Translator::translate($formField->getLabel()) . ': ' . Translator::translate('This field is required.');
                    continue;
                }
                foreach ($formField->getValidators() as $validator) {
                    if (!$validator->validate($value)) {
                        if ($validator instanceof IbanRegexValidator) {
                            $message = sprintf(Translator::translate($validator->getMessage()), Translator::translate('LT01234567891011121314'));
                        } else {
                            $message = Translator::translate($validator->getMessage());
                        }
                        $this->validationErrors[] = Translator::translate($formField->getLabel()) . ': ' . $message;
                    }
                }
            }
        }

        return !$this->validationErrors;
    }

    public function populate()
    {
        $values = [];
        foreach (GatewaySettings::getAllKeys() as $key) {
            $values[$key] = Configuration::get($key);
        }

        //set default values
        foreach ($this->subforms as $subform) {
            foreach ($subform->getFormFields() as $formField) {
                $value = $values[$formField->getName()];
                if ($value === false) {
                    $values[$formField->getName()] = $formField->getDefault();
                }
            }
        }

        $this->tpl_vars['fields_value'] = $values;
    }

    private function getGatewayInfo(Gateway $gateway)
    {
        $html = '';

        if (!$gateway->getKevinProjectSettings()) {
            $html .= AdminHtmlHelper::htmlError(Translator::translate('Incorrect client credentials.'));
        }
        if ($gateway->isSandbox()) {
            $html .= AdminHtmlHelper::htmlWarning(Translator::translate('SME Bank payment gateway is in test mode for testing purposes. Real payments are not available.'));
        }
        $paymentMethodMessage = '';
        if ($gateway->isBankPaymentEnabled()) {
            $paymentMethodMessage .= Translator::translate('Bank payment method is available.') . ' ';
        }
        if ($gateway->isCardPaymentEnabled()) {
            $paymentMethodMessage .= Translator::translate('Card payment method is available.');
        }
        if ($paymentMethodMessage) {
            $html .= AdminHtmlHelper::htmlInfo($paymentMethodMessage);
        }
        $refundMethodMessage = '';
        if ($gateway->isRefundByBankEnabled()) {
            $refundMethodMessage .= Translator::translate('Bank refunds are allowed.') . ' ';
        }
        if ($gateway->isRefundByCardEnabled()) {
            $refundMethodMessage .= Translator::translate('Card refunds are allowed.');
        }
        if ($refundMethodMessage) {
            $html .= AdminHtmlHelper::htmlInfo($refundMethodMessage);
        }

        return $html;
    }
}
