<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */

namespace SmebankPlugin;

if (!defined('_PS_VERSION_')) {
    exit;
}

use Context;
use SmebankPluginFramework\Application\Exception\KevinInitException;
use SmebankPluginFramework\Application\Exception\KevinRequestException;
use SmebankPluginFramework\Application\PaymentClient;
use SmebankPluginFramework\Application\PaymentClientSettings;

class Gateway
{
    /**
     * @var PaymentClient
     */
    private $kevinClient;
    /**
     * @var GatewaySettings
     */
    private $gatewaySettings;

    /**
     * @var array|false
     * false if not initiated
     */
    private $kevinProjectSettings = false;

    public function __construct(PaymentClient $kevinClient, GatewaySettings $gatewaySettings)
    {
        $this->kevinClient = $kevinClient;
        $this->gatewaySettings = $gatewaySettings;
    }

    /**
     * @param GatewaySettings $gatewaySettings
     *
     * @return Gateway
     *
     * @throws KevinInitException
     */
    public static function init(GatewaySettings $gatewaySettings)
    {
        $clientSettings = new PaymentClientSettings(
            PluginConfig::PLUGIN_VERSION(),
            'PrestaShop',
            _PS_VERSION_,
            PluginConfig::SMEBANK_API_DOMAIN(),
            Context::getContext()->language->iso_code
        );
        $client = new PaymentClient($gatewaySettings->getClientId(), $gatewaySettings->getClientSecret(), $clientSettings);

        return new self($client, $gatewaySettings);
    }

    /**
     * @throws KevinRequestException
     */
    public function initPayment(array $attr)
    {
        return $this->kevinClient->initPayment($attr);
    }

    /**
     * @throws KevinRequestException
     */
    public function initPaymentRefund($paymentId, array $attr)
    {
        return $this->kevinClient->initiatePaymentRefund($paymentId, $attr);
    }

    /**
     * @param $paymentId
     *
     * @return array
     *
     * @throws KevinRequestException
     */
    public function getPayment($paymentId)
    {
        return $this->kevinClient->getPayment($paymentId);
    }

    /**
     * @return array
     */
    public function getKevinProjectSettings()
    {
        if ($this->kevinProjectSettings === false) {
            try {
                $this->kevinProjectSettings = $this->kevinClient->getProjectSettings();
            } catch (KevinRequestException $e) {
                $this->kevinProjectSettings = [];
            }
        }

        return $this->kevinProjectSettings;
    }

    /**
     * @return bool
     */
    public function isSandbox()
    {
        return !empty($this->getKevinProjectSettings()['isSandbox']);
    }

    /**
     * @return bool
     */
    public function isBankPaymentEnabled()
    {
        if (empty($this->getKevinProjectSettings()['paymentMethods'])) {
            return false;
        }

        return in_array('bank', $this->getKevinProjectSettings()['paymentMethods']);
    }

    /**
     * @return bool
     */
    public function isCardPaymentEnabled()
    {
        if (empty($this->getKevinProjectSettings()['paymentMethods'])) {
            return false;
        }

        return in_array('card', $this->getKevinProjectSettings()['paymentMethods']);
    }

    /**
     * @return bool
     */
    public function isRefundAllowed()
    {
        return !empty($this->getKevinProjectSettings()['allowedRefundsFor']);
    }

    /**
     * @return bool
     */
    public function isRefundByBankEnabled()
    {
        if (empty($this->getKevinProjectSettings()['allowedRefundsFor'])) {
            return false;
        }

        return in_array('bank', $this->getKevinProjectSettings()['allowedRefundsFor']);
    }

    /**
     * @return bool
     */
    public function isRefundByCardEnabled()
    {
        if (empty($this->getKevinProjectSettings()['allowedRefundsFor'])) {
            return false;
        }

        return in_array('card', $this->getKevinProjectSettings()['allowedRefundsFor']);
    }

    /**
     * @return array
     */
    public function getKevinBanks()
    {
        try {
            return $this->kevinClient->getAllBanks();
        } catch (KevinRequestException $e) {
            return [];
        }
    }

    public function getKevinBanksByCountry($countryCode)
    {
        try {
            return $this->kevinClient->getBanks($countryCode);
        } catch (KevinRequestException $e) {
            return [];
        }
    }

    public function getKevinBank($bankId)
    {
        try {
            return $this->kevinClient->getBank($bankId);
        } catch (KevinRequestException $e) {
            return [];
        }
    }

    /**
     * @return array
     */
    public function getKevinCountries()
    {
        try {
            return $this->kevinClient->getSupportedCountryCodes();
        } catch (KevinRequestException $e) {
            return [];
        }
    }

    public function isEnabled()
    {
        return $this->gatewaySettings->isEnabled() && $this->getKevinProjectSettings();
    }
}
