<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */

namespace SmebankPlugin\Handler;

if (!defined('_PS_VERSION_')) {
    exit;
}

use Cart;
use Context;
use Currency;
use Customer;
use SmebankPlugin\Gateway;
use SmebankPlugin\GatewaySettings;
use SmebankPlugin\Repository\KevinPaymentRepository;
use SmebankPlugin\Utils\Url;
use Validate;

class PaymentInitiationHandler
{
    /**
     * @var GatewaySettings
     */
    private $gatewaySettings;
    /**
     * @var Gateway
     */
    private $gateway;
    /**
     * @var KevinPaymentRepository
     */
    private $kevinPaymentRepository;

    public function __construct(Gateway $gateway, GatewaySettings $gatewaySettings, KevinPaymentRepository $kevinPaymentRepository)
    {
        $this->gatewaySettings = $gatewaySettings;
        $this->gateway = $gateway;
        $this->kevinPaymentRepository = $kevinPaymentRepository;
    }

    /**
     * @param Cart $cart
     * @param $bankId
     *
     * @return string|null
     *
     * @throws \PrestaShopException
     */
    public function handle(Cart $cart, $bankId)
    {
        $customer = new Customer($cart->id_customer);
        if (!Validate::isLoadedObject($customer)) {
            return null;
        }

        $paymentAmount = (float) $cart->getOrderTotal();
        $paymentCurrency = new Currency($cart->id_currency);

        if ($paymentAmount <= 0) {
            return null;
        }

        try {
            $paymentRequest = $this->createKevinPaymentRequest($cart, $customer, $paymentAmount, $paymentCurrency, $bankId);
            $paymentResponse = $this->gateway->initPayment($paymentRequest);
        } catch (\Exception $e) {
            return null;
        }

        $this->kevinPaymentRepository->add($paymentResponse['id'], $paymentResponse['statusGroup'], $bankId, $cart->id);

        return $paymentResponse['confirmLink'];
    }

    private function createKevinPaymentRequest(
        Cart $cart,
        Customer $customer,
        $paymentAmount,
        Currency $paymentCurrency,
        $bankId
    ) {

        $paymentDescription = 'CART ID: ' . $cart->id;

        $attr = [
            'amount' => number_format((float) $paymentAmount, 2, '.', ''),
            'currencyCode' => $paymentCurrency->iso_code,
            'description' => $paymentDescription,
            'identifier' => ['email' => $customer->email],
            'redirectPreferred' => true,
            'Redirect-URL' => Context::getContext()->link->getModuleLink(\Smebank::MODULE_NAME, 'confirm'),
            'Webhook-URL' => Url::getWebhookUrl(),
        ];

        $attr['bankPaymentMethod'] = [
            'creditorName' => $this->gatewaySettings->getCreditorName(),
            'endToEndId' => $paymentDescription,
            'creditorAccount' => [
                'iban' => $this->gatewaySettings->getCreditorAccount(),
            ],
        ];

        if ($bankId === 'card') { //card
            $attr['cardPaymentMethod'] = [];
            $attr['paymentMethodPreferred'] = 'card';
        } elseif ($bankId) { //bank
            $attr['bankId'] = $bankId;
        } else { //frame
            $attr['paymentMethodPreferred'] = 'bank';
        }

        return $attr;
    }
}
