<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */

namespace SmebankPlugin\Handler;

if (!defined('_PS_VERSION_')) {
    exit;
}

use Cart;
use Configuration;
use SmebankPlugin\Repository\KevinPaymentRepository;
use SmebankPlugin\Repository\PsOrderRepository;
use SmebankPluginFramework\Application\KevinConstants;
use Order;

class PaymentWebhookHandler implements WebhookHandlerInterface
{
    /**
     * @var KevinPaymentRepository
     */
    private $kevinPaymentRepository;
    /**
     * @var PsOrderRepository
     */
    private $psOrderRepository;
    /**
     * @var OrderCreationHandler
     */
    private $orderCreationHandler;

    public function __construct(OrderCreationHandler $orderCreationHandler, KevinPaymentRepository $kevinPaymentRepository, PsOrderRepository $psOrderRepository)
    {
        $this->kevinPaymentRepository = $kevinPaymentRepository;
        $this->psOrderRepository = $psOrderRepository;
        $this->orderCreationHandler = $orderCreationHandler;
    }

    public function handle($request)
    {
        $paymentId = $request['id'];
        $kevinStatus = $request['statusGroup'];
        $kevinPayment = $this->kevinPaymentRepository->getByPaymentId($paymentId);

        if (!$kevinPayment) {
            return [
                'isSuccess' => false,
                'message' => 'Payment not found',
            ];
        }

        $cartId = (int)$kevinPayment['id_cart'];
        $orderId = (int)$kevinPayment['id_order'];

        if ($orderId) {
            $order = new Order($orderId);
        } elseif ($this->kevinPaymentRepository->tryLockPayment($paymentId)) {

            if ($kevinStatus === KevinConstants::STATUS_GROUP_FAILED) {
                return [
                    'isSuccess' => true,
                    'message' => 'Order not created for failed payment',
                ];
            }

            $cart = new Cart($cartId);
            $order = $this->orderCreationHandler->handle($cart, $paymentId);
        } else {
            sleep(7);
            return [
                'isSuccess' => false,
                'message' => 'Waiting for order creation..',
            ];
        }

        $this->kevinPaymentRepository->updatePaymentStatus($paymentId, $kevinStatus);

        if ($this->psOrderRepository->isOrderStateInHistory($order->id, (int) Configuration::get('PS_OS_PAYMENT'))) {
            return [
                'isSuccess' => true,
                'message' => 'Order already paid',
            ];
        }

        if ($order->current_state == Configuration::get('PS_OS_CANCELED)')) {
            return [
                'isSuccess' => true,
                'message' => 'Order cancelled',
            ];
        }

        if ($kevinStatus === KevinConstants::STATUS_GROUP_COMPLETED) {
            $order->setCurrentState(Configuration::get('PS_OS_PAYMENT'));
        } elseif ($kevinStatus === KevinConstants::STATUS_GROUP_FAILED) {
            $order->setCurrentState(Configuration::get('PS_OS_ERROR'));
        }

        return [
            'isSuccess' => true,
            'message' => 'Signatures match.',
        ];
    }
}
