<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */

namespace SmebankPlugin\View\Payment17;

if (!defined('_PS_VERSION_')) {
    exit;
}

use Address;
use Cart;
use Context;
use Country;
use Smebank;
use SmebankPlugin\Gateway;
use SmebankPlugin\GatewaySettings;
use SmebankPlugin\Utils\Translator;
use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

class BankPaymentOptionBuilder
{
    /**
     * @param Gateway $gateway
     * @param GatewaySettings $gatewaySettings
     * @param Cart $cart
     *
     * @return PaymentOption
     *
     * @throws \PrestaShopDatabaseException
     * @throws \PrestaShopException
     * @throws \SmartyException
     */
    public static function build(Gateway $gateway, GatewaySettings $gatewaySettings, Cart $cart)
    {
        $address = new Address($cart->id_address_invoice);
        $country = new Country($address->id_country, $cart->id_lang);

        $availableCountries = self::formatCountryCodesList($gateway->getKevinCountries(), $cart->id_lang);

        $isCountryAvailableInKevin = isset($availableCountries[$country->iso_code]);

        $alerts = [];
        if ($gateway->isSandbox()) {
            $alerts[] = Translator::translate('SME Bank payment gateway is in test mode for testing purposes. Real payments are not available.');
        }
        if (!$gatewaySettings->isCountryDropdown() && !$isCountryAvailableInKevin) {
            $alerts[] = sprintf(
                Translator::translate('No bank is available in your selected country: %s'), $country->name
            );
        }

        $context = Context::getContext();

        $errors = [];
        if ($context->cookie->__isset('kevin_error_no_bank')) {
            $context->cookie->__unset('kevin_error_no_bank');
            $errors[] = Translator::translate('Please select your bank.');
        }

        $context->smarty->assign([
            'alerts' => $alerts,
            'countryDropdown' => $gatewaySettings->isCountryDropdown(),
            'bankSearch' => $gatewaySettings->isBankSearch(),
            'isCheckoutViewTiles' => $gatewaySettings->isCheckoutViewTiles(),
            'countryCodes' => $availableCountries,
            'selectedCountryCode' => $isCountryAvailableInKevin ? $country->iso_code : null,
            'banks' => $gateway->getKevinBanks(),
            'text_bank_search_placeholder' => Translator::translate('Bank name'),
            'text_legal_info' => Translator::translate('By using the payment initiation service provided by Kevin EU, UAB, you confirm that you have read and agree with:'),
            'text_legal_privacy_policy' => Translator::translate('Privacy policy'),
            'text_legal_tos' => Translator::translate('Terms and conditions'),
            'text_select_country' => Translator::translate('Select country'),
            'errors' => $errors,
        ]);

        $option = new PaymentOption();
        $option
            ->setModuleName(Smebank::MODULE_NAME)
            ->setCallToActionText(Translator::translate('Bank payments'))
            ->setAdditionalInformation($context->smarty->fetch('module:smebank/views/templates/front/ps17/bank-payment-options.tpl'))
            ->setAction($context->link->getModuleLink(Smebank::MODULE_NAME, 'redirect', ['id' => ''], true));

        return $option;
    }

    /**
     * @param array $kevinCountryCodes
     * @param int $langId
     *
     * @return array
     */
    private static function formatCountryCodesList($kevinCountryCodes, $langId)
    {
        $countryCodes = [];
        foreach ($kevinCountryCodes as $kevinCountryCode) {
            $countryCodes[$kevinCountryCode] = Country::getNameById($langId, Country::getByIso($kevinCountryCode));
        }
        $countryCodes = array_filter(array_unique($countryCodes));
        asort($countryCodes);

        return $countryCodes;
    }
}
