<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */


namespace SmebankPluginFramework\Application;

use Kevin\Client;
use Kevin\KevinException;
use SmebankPluginFramework\Application\Exception\KevinInitException;
use SmebankPluginFramework\Application\Exception\KevinRequestBadRequestException;
use SmebankPluginFramework\Application\Exception\KevinRequestException;
use SmebankPluginFramework\Application\Exception\KevinRequestInternalServerException;
use SmebankPluginFramework\Application\Exception\KevinRequestUnauthorizedException;

class PaymentClient
{
    /**
     * @var Client
     */
    private $client;

    /**
     * @param string $clientId
     * @param string $clientSecret
     * @param PaymentClientSettings $settings
     * @throws KevinInitException
     */
    public function __construct($clientId, $clientSecret, $settings)
    {
		if (!isset($clientId, $clientSecret)) {
			throw new KevinInitException('missing credentials');
		}

        if (!$settings instanceof PaymentClientSettings) {
			throw new KevinInitException('$settings is of wrong type');
        }

        try {
            $this->client = new Client($clientId, $clientSecret, $settings->toArray());
        } catch (KevinException $e) {
            throw new KevinInitException($e->getMessage());
        }
    }

    /**
     * @param array $attributes
     * @return array
     * @throws KevinRequestBadRequestException
     * @throws KevinRequestInternalServerException
     * @throws KevinRequestUnauthorizedException
     * @throws KevinRequestException
     */
    public function initPayment($attributes)
    {
        try {
            return $this->client->payment()->initPayment($attributes);
        } catch (KevinException $e) {
            throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
        }
    }

    /**
     * @param string $paymentId
     * @return array
     * @throws KevinRequestBadRequestException
     * @throws KevinRequestException
     * @throws KevinRequestInternalServerException
     * @throws KevinRequestUnauthorizedException
     */
    public function getPayment($paymentId)
    {
        try {
            return $this->client->payment()->getPayment($paymentId);
        } catch (KevinException $e) {
            throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
        }
    }

    /**
     * @param string $paymentId
     * @param array $attributes
     * @return array
     * @throws KevinRequestBadRequestException
     * @throws KevinRequestInternalServerException
     * @throws KevinRequestUnauthorizedException
     * @throws KevinRequestException
     */
    public function initiatePaymentRefund($paymentId, $attributes)
    {
        try {
            return $this->client->payment()->initiatePaymentRefund($paymentId, $attributes);
        } catch (KevinException $e) {
            throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
        }
    }

    /**
     * @param string $countryCode
     * @return array
     * @throws KevinRequestBadRequestException
     * @throws KevinRequestInternalServerException
     * @throws KevinRequestUnauthorizedException
     * @throws KevinRequestException
     */
    public function getBanks($countryCode)
    {
        try {
            return $this->client->auth()->getBanks(['countryCode' => $countryCode])['data'];
        } catch (KevinException $e) {
            throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
        }
    }

    /**
     * @return array
     * @throws KevinRequestBadRequestException
     * @throws KevinRequestInternalServerException
     * @throws KevinRequestUnauthorizedException
     * @throws KevinRequestException
     */
    public function getAllBanks()
    {
        try {
            return $this->client->auth()->getBanks()['data'];
        } catch (KevinException $e) {
            throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
        }
    }

    /**
     * @return array
     * @throws KevinRequestBadRequestException
     * @throws KevinRequestInternalServerException
     * @throws KevinRequestUnauthorizedException
     * @throws KevinRequestException
     */
    public function getSupportedCountryCodes()
    {
        try {
            return $this->client->auth()->getCountries()['data'];
        } catch (KevinException $e) {
            throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
        }
    }

    /**
     * @param string $bankCode
     * @return array
     * @throws KevinRequestBadRequestException
     * @throws KevinRequestInternalServerException
     * @throws KevinRequestUnauthorizedException
     * @throws KevinRequestException
     */
    public function getBank($bankCode)
    {
        try {
            return $this->client->auth()->getBank($bankCode);
        } catch (KevinException $e) {
            throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
        }
    }

    /**
     * @return array
     * @throws KevinRequestBadRequestException
     * @throws KevinRequestInternalServerException
     * @throws KevinRequestUnauthorizedException
     * @throws KevinRequestException
     */
    public function getPaymentMethods()
    {
        try {
            return $this->client->auth()->getPaymentMethods()['data'];
        } catch (KevinException $e) {
            throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
        }
    }

	/**
	 * @return array
	 * @throws KevinRequestBadRequestException
	 * @throws KevinRequestInternalServerException
	 * @throws KevinRequestUnauthorizedException
	 * @throws KevinRequestException
	 */
	public function getProjectSettings()
	{
		try {
			return $this->client->auth()->getProjectSettings();
		} catch (KevinException $e) {
			throw PaymentClientRequestExceptionFactory::createFromKevinException($e);
		}
	}
}
